@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION @EXPORT @EXPORT_OK @ISA $CurrentPackage @IncludeLibs $ScanFileRE );
 
-$VERSION   = '1.28';
+$VERSION   = '1.31';
 @EXPORT    = qw( scan_deps scan_deps_runtime );
 @EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime path_to_inc_name );
 
@@ -12,20 +12,25 @@ use Config;
 require Exporter;
 our @ISA = qw(Exporter);
 use constant is_insensitive_fs => (
-    -s $0 
+    -s $0
         and (-s lc($0) || -1) == (-s uc($0) || -1)
         and (-s lc($0) || -1) == -s $0
 );
 
 use version;
-use Cwd ();
 use File::Path ();
 use File::Temp ();
-use File::Spec ();
-use File::Basename ();
 use FileHandle;
 use Module::Metadata;
 
+# NOTE: Keep the following imports exactly as specified, even if the Module::ScanDeps source
+# doesn't reference some of them. See '"use lib" idioms' for the reason.
+use Cwd (qw(abs_path));
+use File::Spec;
+use File::Spec::Functions;
+use File::Basename;
+
+
 $ScanFileRE = qr/(?:^|\\|\/)(?:[^.]*|.*\.(?i:p[ml]|t|al))$/;
 
 =head1 NAME
@@ -172,7 +177,7 @@ returns a reference to it.
     $perl_name = path_to_inc_name($path, $warn)
 
 Assumes C<$path> refers to a perl file and does it's best to return the
-name as it would appear in %INC. Returns undef if no match was found 
+name as it would appear in %INC. Returns undef if no match was found
 and a prints a warning to STDERR if C<$warn> is true.
 
 E.g. if C<$path> = perl/site/lib/Module/ScanDeps.pm then C<$perl_name>
@@ -187,8 +192,8 @@ which to search modules without modifying C<@INC> itself.
 
 =head2 B<$Module::ScanDeps::ScanFileRE>
 
-You can set this global variable to specify a regular expression to 
-identify what files to scan. By default it includes all files of 
+You can set this global variable to specify a regular expression to
+identify what files to scan. By default it includes all files of
 the following types: .pm, .pl, .t and .al. Additionally, all files
 without a suffix are considered.
 
@@ -224,7 +229,7 @@ my %Preload = (
     'AnyDBM_File.pm'                    => [qw( SDBM_File.pm )],
     'AnyEvent.pm'                       => 'sub',
     'Authen/SASL.pm'                    => 'sub',
-    'B/Hooks/EndOfScope.pm'             => 
+    'B/Hooks/EndOfScope.pm'             =>
         [qw( B/Hooks/EndOfScope/PP.pm B/Hooks/EndOfScope/XS.pm )],
     'Bio/AlignIO.pm'                    => 'sub',
     'Bio/Assembly/IO.pm'                => 'sub',
@@ -255,7 +260,7 @@ my %Preload = (
                 _glob_in_inc('Catalyst/DispatchType', 1));
     },
     'Catalyst/Engine.pm'                => 'sub',
-    'CGI/Application/Plugin/Authentication.pm' => 
+    'CGI/Application/Plugin/Authentication.pm' =>
         [qw( CGI/Application/Plugin/Authentication/Store/Cookie.pm )],
     'CGI/Application/Plugin/AutoRunmode.pm' => [qw( Attribute/Handlers.pm )],
     'charnames.pm'                      => \&_unicore,
@@ -269,7 +274,7 @@ my %Preload = (
     'Crypt/Random/Generator.pm'         => sub {
         _glob_in_inc('Crypt/Random/Provider', 1);
     },
-    'Date/Manip.pm'                     => 
+    'Date/Manip.pm'                     =>
         [qw( Date/Manip/DM5.pm Date/Manip/DM6.pm )],
     'Date/Manip/Base.pm'                => sub {
         _glob_in_inc('Date/Manip/Lang', 1);
@@ -290,7 +295,7 @@ my %Preload = (
     'DBIx/Perlish.pm'                   => [qw( attributes.pm )],
     'DBIx/ReportBuilder.pm'             => 'sub',
     'Device/ParallelPort.pm'            => 'sub',
-    'Device/SerialPort.pm'              => 
+    'Device/SerialPort.pm'              =>
         [qw( termios.ph asm/termios.ph sys/termiox.ph sys/termios.ph sys/ttycom.ph )],
     'diagnostics.pm'                    => sub {
         # shamelessly taken and adapted from diagnostics.pm
@@ -312,7 +317,7 @@ my %Preload = (
         ) {
             return $_ if _find_in_inc($_);
         }
-        
+
         for (
               "$archlib/pods/perldiag.pod",
               "$privlib/pods/perldiag-$Config{version}.pod",
@@ -358,6 +363,7 @@ my %Preload = (
         IO/Pipe.pm          IO/Socket.pm        IO/Dir.pm
     )],
     'IO/Socket.pm'                      => [qw( IO/Socket/UNIX.pm )],
+    'IUP.pm'                            => 'sub',
     'JSON.pm'                           => sub {
         # add JSON/PP*.pm, JSON/PP/*.pm
         # and ignore other JSON::* modules (e.g. JSON/Syck.pm, JSON/Any.pm);
@@ -365,7 +371,7 @@ my %Preload = (
         return( grep /^JSON\/(PP|XS)/, _glob_in_inc('JSON', 1) );
     },
     'JSON/MaybeXS.pm'                   => [qw(
-        Cpanel/JSON/XS.pm JSON/XS.pm JSON/PP.pm 
+        Cpanel/JSON/XS.pm JSON/XS.pm JSON/PP.pm
     )],
     'List/MoreUtils.pm'                 => 'sub',
     'List/SomeUtils.pm'                 => 'sub',
@@ -385,7 +391,7 @@ my %Preload = (
     },
     'LWP/Parallel/UserAgent.pm'         => [qw( LWP/Parallel.pm )],
     'LWP/UserAgent.pm'                  => sub {
-        return( 
+        return(
           qw( URI/URL.pm URI/http.pm LWP/Protocol/http.pm ),
           _glob_in_inc("LWP/Authen", 1),
           _glob_in_inc("LWP/Protocol", 1),
@@ -401,6 +407,7 @@ my %Preload = (
     'Module/Pluggable.pm'               => sub {
         _glob_in_inc('$CurrentPackage/Plugin', 1);
     },
+    'Moo.pm'                            => [qw( Class/XSAccessor.pm )],
     'Moose.pm'                          => sub {
         _glob_in_inc('Moose', 1),
         _glob_in_inc('Class/MOP', 1),
@@ -519,17 +526,19 @@ my %Preload = (
         _glob_in_inc('XML/Parser/Encodings', 1),
     },
     'XML/SAX.pm'                        => [qw( XML/SAX/ParserDetails.ini ) ],
+    'XML/Twig.pm'                       => [qw( URI.pm )],      # or URI::File or LWP
+    'XML/Twig/XPath.pm'                 => [qw( XML/XPathEngine.pm XML/XPath.pm )],
     'XMLRPC/Lite.pm'                    => sub {
         _glob_in_inc('XMLRPC/Transport', 1);
     },
     'YAML.pm'                           => [qw( YAML/Loader.pm YAML/Dumper.pm )],
-    'YAML/Any.pm'                       => sub { 
+    'YAML/Any.pm'                       => sub {
         # try to figure out what YAML::Any would have used
         my $impl = eval "use YAML::Any; YAML::Any->implementation;";
-        unless ($@) 
-        { 
-            $impl =~ s!::!/!g; 
-            return "$impl.pm"; 
+        unless ($@)
+        {
+            $impl =~ s!::!/!g;
+            return "$impl.pm";
         }
         _glob_in_inc('YAML', 1);        # fallback
     },
@@ -616,7 +625,51 @@ sub scan_deps {
         }
     }
 
-    scan_deps_static(\%args);
+    {
+        ## "use lib" idioms
+        #
+        # We want to correctly interprete stuff like
+        #
+        #   use FindBin;
+        #   use lib "$FindBin/../lib";
+        #
+        # Find out what $FindBin::Bin etc would have been if "use FindBin" had been
+        # called in the first file to analyze.
+        #
+        # Notes:
+        # (1) We don't want to reimplement FindBin, hence fake $0 locally (as the path of the
+        #     first file analyzed) and call FindBin::again().
+        # (2) If the caller of scan_deps() itself uses FindBin, we don't want to overwrite
+        #     the value of "their" $FindBin::Bin.
+        #
+        # Other idioms seen sometimes:
+        #
+        # use lib "$ENV{FOO}/path";
+        # use lib File::Spec->catdir($FindBin::Bin, qw[.. qqlib] );
+        # use lib catdir(dirname($0), "perl");
+        # use lib dirname(abs_path($0));
+        #
+        # In order to correctly interprete these, the modules referenced have to be imported.
+
+
+        require FindBin;
+
+        local $FindBin::Bin;
+        local $FindBin::RealBin;
+        local $FindBin::Script;
+        local $FindBin::RealScript;
+
+        my $_0 = $args{files}[0];
+        local *0 = \$_0;
+        FindBin->again();
+
+        our $Bin = $FindBin::Bin;
+        our $RealBin = $FindBin::RealBin;
+        our $Script = $FindBin::Script;
+        our $RealScript = $FindBin::RealScript;
+
+        scan_deps_static(\%args);
+    }
 
     if ($args{execute} or $args{compile}) {
         scan_deps_runtime(
@@ -678,7 +731,7 @@ sub scan_deps_static {
         }else{ # no caching callback given
             @pm = scan_file($file);
         }
-        
+
         foreach my $pm (@pm){
             add_deps(
                      used_by => $key,
@@ -704,7 +757,7 @@ sub scan_deps_static {
 
     # prevent utf8.pm from being scanned
     $_skip->{$rv->{"utf8.pm"}{file}}++ if $rv->{"utf8.pm"};
-   
+
     while ($recurse) {
         my $count = keys %$rv;
         my @files = sort grep { defined $_->{file} && -T $_->{file} } values %$rv;
@@ -803,7 +856,7 @@ sub scan_line {
     foreach (split(/;/, $line)) {
         s/^\s*//;
         #  handle single line blocks like 'do { package foo; use xx; }'
-        s/^(?:do\s*)?\{\s*//;  
+        s/^(?:do\s*)?\{\s*//;
         s/\}$//;
 
         if (/^package\s+(\w+)/) {
@@ -827,13 +880,13 @@ sub scan_line {
             # be specified for the "autouse" and "if" pragmas, e.g.
             #   use autouse Module => qw(func1 func2);
             #   use autouse "Module", qw(func1);
-            # To avoid to parse them ourself, we simply try to eval the 
+            # To avoid to parse them ourself, we simply try to eval the
             # string after the pragma (in a list context). The MODULE
             # should be the first ("autouse") or second ("if") element
             # of the list.
             my $module;
-            { 
-                no strict; no warnings; 
+            {
+                no strict; no warnings;
                 if ($pragma eq "autouse") {
                     ($module) = eval $args;
                 }
@@ -841,7 +894,7 @@ sub scan_line {
                     # The syntax of the "if" pragma is
                     #   use if COND, MODULE => ARGUMENTS
                     # The COND may contain undefined functions (i.e. undefined
-                    # in Module::ScanDeps' context) which would throw an 
+                    # in Module::ScanDeps' context) which would throw an
                     # exception. Sneak  "1 || " in front of COND so that
                     # COND will not be evaluated. This will work in most
                     # cases, but there are operators with lower precedence
@@ -864,7 +917,7 @@ sub scan_line {
             foreach my $dir (do { no strict; no warnings; eval $libs }) {
                 next unless defined $dir;
                 my @dirs = $dir;
-                push @dirs, "$dir/$ver", "$dir/$archname", "$dir/$ver/$archname" 
+                push @dirs, "$dir/$ver", "$dir/$archname", "$dir/$ver/$archname"
                     if $how =~ /lib/;
                 foreach (@dirs) {
                     unshift(@INC, $_) if -d $_;
@@ -884,7 +937,7 @@ my %LoaderRegexp; # cache
 sub _build_loader_regexp {
     my $loaders = shift;
     my $prefix = (@_ && $_[0]) ? $_[0].'::' : '';
-   
+
     my $loader = join '|', map quotemeta($_), split /\s+/, $loaders;
     my $regexp = qr/^\s* use \s+ ($loader)(?!\:) \b \s* (.*)/sx;
     # WARNING: This doesn't take the prefix into account
@@ -919,7 +972,7 @@ sub scan_chunk {
         $_ = $chunk;
         s/^\s*//;
         #  handle single line blocks like 'do { package foo; use xx; }'
-        s/^(?:do\s*)?\{\s*//;  
+        s/^(?:do\s*)?\{\s*//;
         s/\}\s*$//;
 
         # TODO: There's many more of these "loader" type modules on CPAN!
@@ -968,12 +1021,15 @@ sub scan_chunk {
             my @mods;
             push @mods, qw( PerlIO.pm PerlIO/encoding.pm Encode.pm ), _find_encoding($1)
                 if $args =~ /:encoding\((.*?)\)/;
-            push @mods, qw( PerlIO.pm PerlIO/via.pm )
-                if $args =~ /:via\(/;
+            while ($args =~ /:(\w+)(?:\((.*?)\))?/g) {
+                push @mods, "PerlIO/$1.pm";
+                push @mods, "Encode.pm", _find_encoding($2) if $1 eq "encoding";
+            }
+            push @mods, "PerlIO.pm" if @mods;
             return \@mods if @mods;
         }
         if (/\b(?:en|de)code\(\s*['"]?([-\w]+)/) {
-            return [qw( Encode.pm ), _find_encoding($1)]; 
+            return [qw( Encode.pm ), _find_encoding($1)];
         }
 
         return $1 if /\b do \s+ ([\w:\.\-\\\/\"\']*)/x;
@@ -1042,7 +1098,7 @@ sub _add_info {
     $file = File::Spec->rel2abs($file);
     $file =~ s|\\|\/|go;
 
-    # Avoid duplicates that can arise due to case differences that don't actually 
+    # Avoid duplicates that can arise due to case differences that don't actually
     # matter on a case tolerant system
     if (File::Spec->case_tolerant()) {
         foreach my $key (keys %$rv) {
@@ -1076,7 +1132,7 @@ sub _add_info {
           if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $used_by } @{ $rv->{$module}{used_by} })
              or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($used_by) } @{ $rv->{$module}{used_by} }));
 
-        # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}    
+        # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}
         push @{ $rv->{$used_by}{uses} }, $module
           if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $module } @{ $rv->{$used_by}{uses} })
              or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($module) } @{ $rv->{$used_by}{uses} }));
@@ -1187,7 +1243,7 @@ sub _glob_in_inc {
 # like _glob_in_inc, but looks only at the first level
 # (i.e. the children of $subdir)
 # NOTE: File::Find has no public notion of the depth of the traversal
-# in its "wanted" callback, so it's not helpful 
+# in its "wanted" callback, so it's not helpful
 sub _glob_in_inc_1 {
     my $subdir  = shift;
     my $pm_only = shift;
@@ -1199,7 +1255,7 @@ sub _glob_in_inc_1 {
         my $dir = "$inc/$subdir";
         next unless -d $dir;
 
-        opendir(my $dh, $dir) or next; 
+        opendir(my $dh, $dir) or next;
         my @names = map { "$subdir/$_" } grep { -f "$dir/$_" } readdir $dh;
         closedir $dh;
 
@@ -1343,10 +1399,10 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
 ...
 
     # NOTE: When compiling the block will run as the last CHECK block;
-    # when executing the block will run as the first END block and 
+    # when executing the block will run as the first END block and
     # the programs continues.
-    print $ih 
-        $execute ? "END\n" : "CHECK\n", 
+    print $ih
+        $execute ? "END\n" : "CHECK\n",
         <<'...';
 {
     require DynaLoader;
@@ -1389,12 +1445,12 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
     @_INC = grep { !ref $_ } @_INC;
 
     my @dlls = grep { defined $_ && -e $_ } Module::ScanDeps::DataFeed::_dl_shared_objects();
-    my @shared_objects = @dlls; 
-    push @shared_objects, grep { s/\.\Q$dlext\E$/.bs/ && -e $_ } @dlls;
+    my @shared_objects = @dlls;
+    push @shared_objects, grep { -e $_ } map { (my $bs = $_) =~ s/\.\Q$dlext\E$/.bs/; $bs } @dlls;
 
     # write data file
     my $data_file = $ENV{MSD_DATA_FILE};
-    open my $fh, ">", $data_file 
+    open my $fh, ">", $data_file
         or die "Couldn't open $data_file: $!\n";
     print $fh Data::Dumper::Dumper(
     {
@@ -1452,11 +1508,11 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         $execute ? @$execute : ());
 
     die $execute
-        ? "SYSTEM ERROR in executing $file @$execute: $rc" 
-        : "SYSTEM ERROR in compiling $file: $rc" 
+        ? "SYSTEM ERROR in executing $file @$execute: $rc"
+        : "SYSTEM ERROR in compiling $file: $rc"
         unless $rc == 0;
-    
-    my $info = do $data_file 
+
+    my $info = do $data_file
         or die "error extracting info from -c/-x file: ", ($@ || "can't read $data_file: $!");
 
     return $info;
@@ -1468,8 +1524,9 @@ sub _info2rv {
 
     my $rv = {};
 
-    my $incs = join('|', sort { length($b) <=> length($a) } 
-                              map { s:\\:/:g; quotemeta($_) } @{ $info->{'@INC'} });
+    my $incs = join('|', sort { length($b) <=> length($a) }
+                              map { s:\\:/:g; s:^(/.*?)/+$:$1:; quotemeta($_) }
+                                  @{ $info->{'@INC'} });
     my $i = is_insensitive_fs() ? "i" : "";
     my $strip_inc_prefix = qr{^(?$i:$incs)/};
 
@@ -1521,8 +1578,8 @@ sub _merge_rv {
         my %mark;
         if ($rv->{$key} and _not_dup($key, $rv, $rv_sub)) {
             warn "Different modules for file '$key' were found.\n"
-                . " -> Using '" . _abs_path($rv_sub->{$key}{file}) . "'.\n"
-                . " -> Ignoring '" . _abs_path($rv->{$key}{file}) . "'.\n";
+                . " -> Using '" . abs_path($rv_sub->{$key}{file}) . "'.\n"
+                . " -> Ignoring '" . abs_path($rv->{$key}{file}) . "'.\n";
             $rv->{$key}{used_by} = [
                 grep (!$mark{$_}++,
                     @{ $rv->{$key}{used_by} },
@@ -1555,22 +1612,13 @@ sub _merge_rv {
 sub _not_dup {
     my ($key, $rv1, $rv2) = @_;
     if (File::Spec->case_tolerant()) {
-        return lc(_abs_path($rv1->{$key}{file})) ne lc(_abs_path($rv2->{$key}{file}));
+        return lc(abs_path($rv1->{$key}{file})) ne lc(abs_path($rv2->{$key}{file}));
     }
     else {
-        return _abs_path($rv1->{$key}{file}) ne _abs_path($rv2->{$key}{file});
+        return abs_path($rv1->{$key}{file}) ne abs_path($rv2->{$key}{file});
     }
 }
 
-sub _abs_path {
-    return join(
-        '/',
-        Cwd::abs_path(File::Basename::dirname($_[0])),
-        File::Basename::basename($_[0]),
-    );
-}
-
-
 sub _warn_of_runtime_loader {
     my $module = shift;
     return if $SeenRuntimeLoader{$module}++;
