@@ -9,23 +9,27 @@ use Module::ScanDeps;
 use ExtUtils::MakeMaker;
 use subs qw( _name _modtree );
 
+my $usage = "Usage: $0 [ -B ] [ -V ] [ -T ] [ -x [ --xargs STRING ] | -c ] [ -R ] [-C FILE ] [ -e STRING | FILE ... ]\n";
+
 my %opts;
 GetOptions(\%opts,
     "B|bundle",
     "C|cachedeps=s",
-    "c|compile".
+    "c|compile",
     "e|eval=s",
     "xargs=s",
     "R|no-recurse",
+    "T|modtree",
     "V|verbose",
     "x|execute",
-);
+) or die $usage;
 
 my (%map, %skip);
 my $core    = $opts{B};
 my $verbose = $opts{V};
 my $eval    = $opts{e};
 my $recurse = $opts{R} ? 0 : 1;
+my $modtree = {} unless $opts{T}; # i.e. disable it unless explicitly requested
 
 if ($eval) {
     require File::Temp;
@@ -40,7 +44,7 @@ if ($opts{x} && defined $opts{xargs}) {
     $opts{x} = [ Text::ParseWords::shellwords($opts{xargs}) ];
 }
 
-die "Usage: $0 [ -B ] [ -V ] [ -x | -c ] [ -R ] [-C FILE ] [ -e STRING | FILE ... ]\n" unless @ARGV;
+die $usage unless @ARGV;
 
 my @files = @ARGV;
 while (<>) {
@@ -79,8 +83,8 @@ foreach my $key (sort keys %$map) {
 
     next if $skip{$name};
 
-    my $privPath = "$Config::Config{privlibexp}/$key";
-    my $archPath = "$Config::Config{archlibexp}/$key";
+    my $privPath = "$Config{privlibexp}/$key";
+    my $archPath = "$Config{archlibexp}/$key";
     $privPath =~ s|\\|\/|og;
     $archPath =~ s|\\|\/|og;
     if ($mod->{file} eq $privPath
@@ -101,13 +105,9 @@ foreach my $key (sort keys %$map) {
 
 $len += 2;
 
-print "# Legend: [C]ore [X]ternal [S]ubmodule [?]NotOnCPAN\n" if $verbose;
-
-foreach my $mod (sort {
-    "@{$a->{used_by}}" cmp "@{$b->{used_by}}" or
-    $a->{key} cmp $b->{key}
-} @todo) {
+print "#\n# Legend: [C]ore [X]ternal [S]ubmodule [?]NotOnCPAN\n" if $verbose;
 
+foreach my $mod (sort { $a->{name} cmp $b->{name} } @todo ) {
     my $version = MM->parse_version($mod->{file});
 
     if (!$verbose) {
@@ -136,7 +136,6 @@ sub _name {
     return $str;
 }
 
-my $modtree;
 sub _modtree {
     $modtree ||= eval {
         require CPANPLUS::Backend;
@@ -167,7 +166,8 @@ scandeps.pl - Scan file prerequisites
 F<scandeps.pl> is a simple-minded utility that prints out the
 C<PREREQ_PM> section needed by modules.
 
-If you have B<CPANPLUS> installed, modules that are part of an
+If the option C<-T> is specified and
+you have B<CPANPLUS> installed, modules that are part of an
 earlier module's distribution with be denoted with C<S>; modules
 without a distribution name on CPAN are marked with C<?>.
 
@@ -200,11 +200,12 @@ Compiles the code and inspects its C<%INC>, in addition to static scanning.
 =item B<-x>, B<--execute>
 
 Executes the code and inspects its C<%INC>, in addition to static scanning.
+You may use B<--xargs> to specify C<@ARGV> when executing the code.
 
 =item B<--xargs>=I<STRING>
 
-If B<-x> is given, splits the C<STRING> using the function 
-C<shellwords> from L<Text::ParseWords> and passes the result 
+If B<-x> is given, splits the C<STRING> using the function
+C<shellwords> from L<Text::ParseWords> and passes the result
 as C<@ARGV> when executing the code.
 
 =item B<-B>, B<--bundle>
@@ -217,7 +218,7 @@ Only show dependencies found in the files listed and do not recurse.
 
 =item B<-V>, B<--verbose>
 
-Verbose mode: Output all files found during the process; 
+Verbose mode: Output all files found during the process;
 show dependencies between modules and availability.
 
 Additionally, warns of any missing dependencies. If you find missing
@@ -229,6 +230,10 @@ false positives.
 Use CACHEFILE to speed up the scanning process by caching dependencies.
 Creates CACHEFILE if it does not exist yet.
 
+=item B<-T>, B<--modtree>
+
+Retrieves module information from CPAN if you have B<CPANPLUS> installed.
+
 =back
 
 =head1 SEE ALSO
